import { AppConfig } from "./app-config.model";
import { Etcd3 } from "etcd3";
import { load } from "js-yaml";
import { hostname } from "os";
import { getEtcdInstance } from "./etcd-connection";
import { readFile } from "fs/promises";
import debug from "debug";
import { findUp } from "./utils/find-up";

export class ConfigLoader {
  private etcd: Etcd3;

  private readonly currHost = hostname();
  private readonly configKeyPrefix = "share/config";
  private readonly logger = debug("fennec:config:loader");

  constructor(config?: AppConfig) {
    this.etcd = getEtcdInstance(config);
  }

  async load<T extends Object>(configKey?: string): Promise<any> {
    const config = await findUp(
      ["config.yml", "config.yaml"],
      process.cwd()
    ).then((path) => {
      if (path) {
        return readFile(path, "utf8").then((str) => load(str));
      } else {
        return undefined;
      }
    });

    if (configKey === undefined) {
      if (process.env.CONFIG_KEY) {
        configKey = process.env.CONFIG_KEY;
      } else if (process.env.config_key) {
        configKey = process.env.config_key;
      } else {
        configKey = await findUp("package.json", process.cwd())
          .then((path) => {
            if (path) {
              return readFile(path, "utf8");
            } else {
              throw new Error(
                "Can't find package.json. Fallback to using package name as configuration name when CONFIG_KEY not defined."
              );
            }
          })
          .then((json) => JSON.parse(json)?.name as string);
      }
    }
    const etcdConfigKey = `${this.configKeyPrefix}/${this.currHost}/${configKey}`;
    const str = await this.etcd.get(etcdConfigKey).string();

    if (!str || !config) {
      this.logger(
        `Not define configuration.
 - Put configuration in %s
 - Provide configuration key in CONFIG_KEY environment variable
 - Create "config.yml" file in current working directory (CWD)`,
        etcdConfigKey
      );
      throw new Error(`Not define configuration.`);
    }
    return Object.assign(config, load(str)) as T;
  }
}
